/* 
  IG Testsensor based on 
  Jeti Sensor EX Telemetry C++ Library
  
  Main program
  --------------------------------------------------------------------
  
  Copyright (C) 2015 Bernd Wokoeck

  *** Extended notice on additional work and copyrights, see header of JetiExProtocol.cpp ***

  Wiring:

    Arduino Mini  TXD-Pin 0 <-- Receiver "Ext." input (orange cable)

  Ressources:
    Uses built in UART of Arduini Mini Pro 328 
  
  Version history:
  0.90   11/22/2015  created
  0.95   12/23/2015  new sample sensors for GPS and date/time
  0.96   02/21/2016  comPort number as optional parameter for Teensy in Start(...)
                     sensor device id as optional parameter (SetDeviceId(...))
  0.99   06/05/2016  max number of sensors increased to 32 (set MAX_SENSOR to a higher value in JetiExProtocol.h if you need more)
                     bug with TYPE_6b removed
                     DemoSensor delivers 18 demo values now
  1.00   01/29/2017  Some refactoring:
                     - Bugixes for Jetibox keys and morse alarms (Thanks to Ingmar !)
                     - Optimized half duplex control for AVR CPUs in JetiExHardwareSerialInt class (for improved Jetibox key handling)
                     - Reduced size of serial transmit buffer (128-->64 words) 
                     - Changed bitrates for serial communication for AVR CPUs (9600-->9800 bps)
                     - EX encryption removed, as a consequence: new manufacturer ID: 0xA409
                       *** Telemetry setup in transmitter must be reconfigured (telemetry reset) ***
                     - Delay at startup increased (to give receiver more init time)
                     - New HandleMenu() function in JetiexSensor.ini (including more alarm test)
                     - JETI_DEBUG and BLOCKING_MODE removed (cleanup)

  Permission is hereby granted, free of charge, to any person obtaining
  a copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.

**************************************************************/
// Include the Bounce2 library found here :
// https://github.com/thomasfredericks/Bounce2
#include <Bounce2.h>

#include <Arduino.h>
#include "JetiExProtocol.h"

JetiExProtocol jetiEx;

void HandleMenu();

enum
{
//  ID_Speed_Kmh = 1,
//  ID_Speed_ms,
  ID_001 = 1,
  ID_002,
  ID_003,
  ID_004,
  ID_005,
  ID_006,
  ID_007,
  ID_008,
  ID_009,
  ID_010,
  ID_011,
  ID_012,
  ID_013,
  ID_014,
  ID_015,
  ID_016,
  ID_017,
  ID_018,
  ID_019,
  ID_020,
  ID_021,
  ID_022,
  ID_023,
  ID_024,
  ID_025,
  ID_026,
  ID_027,
  ID_028,
  ID_029,
  ID_030,
  ID_031,
  ID_032
};

// precision = 0 --> 0, precision = 1 --> 0.0, precision = 2 --> 0.00
JETISENSOR_PTR sensors[] = 
{
  //              id             name        unit    data type          precision 
//  new JetiSensor( ID_Speed_Kmh,  "Speed",    "Km/h",  JetiSensor::TYPE_14b, 0 ),
//  new JetiSensor( ID_Speed_ms,   "Speed",    "m/s",   JetiSensor::TYPE_14b, 1 ),
  new JetiSensor( ID_001,  "001",    "",  JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_002,   "002",    "",   JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_003,    "003",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_004,    "004",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_005,    "005",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_006,    "006",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_007,    "007",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_008,    "008",    "",     JetiSensor::TYPE_14b, 0 ),
/*
  new JetiSensor( ID_009,    "009",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_010,    "010",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_011,    "011",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_012,    "012",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_013,    "013",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_014,    "014",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_015,    "015",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_016,    "016",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_017,    "017",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_018,    "018",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_019,    "019",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_020,    "020",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_021,    "021",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_022,    "022",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_023,    "023",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_024,    "024",    "",     JetiSensor::TYPE_14b, 0 ),

  new JetiSensor( ID_025,    "025",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_026,    "026",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_027,    "027",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_028,    "028",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_029,    "029",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_030,    "030",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_031,    "031",    "",     JetiSensor::TYPE_14b, 0 ),
  new JetiSensor( ID_032,    "032",    "",     JetiSensor::TYPE_14b, 0 ),
*/
  0 // end of array
};

#define PinPushButton 2
#define PinDirection  3
#define PinCLK        4

char cLine1[] = "0123456789ABCDEF";
char cLine2[] = "0123456789ABCDEF";
long lSpeed_Kmh = 0;
long lSpeed_ms = 0;

bool bIncrese = false;
long lSpeed = 0;

long lTestValue = 0;

Bounce PushButton = Bounce(PinPushButton, 50);
Bounce Direction = Bounce(PinDirection, 2);
Bounce Clk = Bounce(PinCLK, 2);

void setup()
{
#ifdef CORE_TEENSY 
  Serial.begin( 9700 ); //IG Jeti-Telemetrie: 9600..9800Bd, optimal:9700
#endif
  jetiEx.SetDeviceId( 0x62, 0x32 ); // 0x3262
  jetiEx.Start( "IG_Testsensor", sensors, JetiExProtocol::SERIAL2 );

  pinMode(PinPushButton, INPUT_PULLUP);
  pinMode(PinDirection, INPUT_PULLUP);
  pinMode(PinCLK, INPUT_PULLUP);

}

void loop()
{
  PushButton.update();
  Direction.update();
  Clk.update();
  
  if(PushButton.fallingEdge()) bIncrese = !bIncrese;
  if(Clk.fallingEdge())
  {
    if(Direction.read())
    { 
      if(bIncrese)
      {
        lSpeed += 10;
      }
      else 
      {
        lSpeed += 1;
      }
    }
    else
    {
      if (bIncrese) 
      {
        if(lSpeed > 10) lSpeed -= 10;
      }
      else 
      {
        if(lSpeed > 0) lSpeed -= 1;
      }
    }
  }

  if(bIncrese) lTestValue = 1000;
  else lTestValue = 0;

  lSpeed_Kmh = (long)((float)lSpeed * 0.36);  // 36 shown as 36Km/h (precision==0)
  lSpeed_ms = lSpeed * 1;                     // 100 shown as 10.0m/s (precision==1)

//  jetiEx.SetSensorValue( ID_Speed_Kmh,  lSpeed_Kmh );
//  jetiEx.SetSensorValue( ID_Speed_ms,  lSpeed_ms );

  jetiEx.SetSensorValue( ID_001,  1 + lTestValue );
  jetiEx.SetSensorValue( ID_002,  2 + lTestValue );
    jetiEx.SetSensorValue( ID_003,  3 + lTestValue );
    jetiEx.SetSensorValue( ID_004,  4 + lTestValue );
    jetiEx.SetSensorValue( ID_005,  5 + lTestValue );
    jetiEx.SetSensorValue( ID_006,  6 + lTestValue );
    jetiEx.SetSensorValue( ID_007,  7 + lTestValue );
    jetiEx.SetSensorValue( ID_008,  8 + lTestValue );
/*  
    jetiEx.SetSensorValue( ID_009,  9 + lTestValue );
    jetiEx.SetSensorValue( ID_010,  10 + lTestValue );
    jetiEx.SetSensorValue( ID_011,  11 + lTestValue );
    jetiEx.SetSensorValue( ID_012,  12 + lTestValue );
    jetiEx.SetSensorValue( ID_013,  13 + lTestValue );
    jetiEx.SetSensorValue( ID_014,  14 + lTestValue );
    jetiEx.SetSensorValue( ID_015,  15 + lTestValue );
    jetiEx.SetSensorValue( ID_016,  16 + lTestValue );
    jetiEx.SetSensorValue( ID_017,  17 + lTestValue );
    jetiEx.SetSensorValue( ID_018,  18 + lTestValue );
    jetiEx.SetSensorValue( ID_019,  19 + lTestValue );
    jetiEx.SetSensorValue( ID_020,  20 + lTestValue );
    jetiEx.SetSensorValue( ID_021,  21 + lTestValue );
    jetiEx.SetSensorValue( ID_022,  22 + lTestValue );
    jetiEx.SetSensorValue( ID_023,  23 + lTestValue );
    jetiEx.SetSensorValue( ID_024,  24 + lTestValue );

    jetiEx.SetSensorValue( ID_025,  25 + lTestValue );
    jetiEx.SetSensorValue( ID_026,  26 + lTestValue );
    jetiEx.SetSensorValue( ID_027,  27 + lTestValue );
    jetiEx.SetSensorValue( ID_028,  28 + lTestValue );
    jetiEx.SetSensorValue( ID_029,  29 + lTestValue );
    jetiEx.SetSensorValue( ID_030,  30 + lTestValue );
    jetiEx.SetSensorValue( ID_031,  31 + lTestValue );
    jetiEx.SetSensorValue( ID_032,  32 + lTestValue );
*/

  HandleMenu();
  digitalWrite(LED_BUILTIN, HIGH);
  jetiEx.DoJetiSend(); 
  digitalWrite(LED_BUILTIN, LOW);
}

void HandleMenu()
{
  static int _x = 0;
  static int _y = 0;
  static int _xLB = 0;
  static int _xUB = 0;
  static int _yLB = 0;
  static int _yUB = 2;
  static uint8_t _cTemp = 0;
  static unsigned long _ulKeyPressed = 0;
  static unsigned long _ulKeyPressedRepeat = 0;
  static unsigned int _uiRepeat = 500;
  static int iSensor = 0;

  if((millis() - _ulKeyPressedRepeat) > 2000) _uiRepeat = 250;
  if((millis() - _ulKeyPressedRepeat) > 5000) _uiRepeat = 100;

  uint8_t c = jetiEx.GetJetiboxKey();
  if((c != _cTemp) || ((millis() - _ulKeyPressed) > _uiRepeat))
  {
    if(c != _cTemp)
    {
      _cTemp = c;
      _uiRepeat = 500;
      _ulKeyPressedRepeat = millis();
    }
    _ulKeyPressed = millis();
    if( c == 0xb0 ) //down
    {
      if( _y < _yUB ) _y++;
    }
    if( c == 0xd0 ) //up
    {
	    if( _y > _yLB ) _y--;
    }
    if( c == 0xe0 ) //right
    {
      if( _x < _xUB ) _x++;
    }
    if( c == 0x70 ) //left
    {
	    if( _x > _xLB ) _x--;
    }
    
    switch(_x) //X menu level
    {
      case 0: //
      {
        switch(_y) //Y menu level
        {
          case 0: //Show Value
          {
            if( c == 0xe0 ) //right
            {
              if( iSensor < 2 ) iSensor++;
            }
            if( c == 0x70 ) //left
            {
              if( iSensor > 0 ) iSensor--;
            }
            sprintf(cLine1, "IG_Testsensor");
            if(iSensor == 0)
            {
              sprintf(cLine2, " Speed:%01dKm/h>", (int)lSpeed_Kmh);
            }               
            if(iSensor == 1) 
            {
              sprintf(cLine2, "<Speed:%01dKm/h>", (int)lSpeed_Kmh);
            }              
            if(iSensor == 2) 
            {
              int iS1 = abs(lSpeed_ms / 10);
              int iS2 = abs(lSpeed_ms % 10);
              sprintf(cLine2, "<Speed:%01d.%01dm/s>", iS1, iS2);
            }              
            break;
          } //case 0
        }
        break;
      } //case 0
      
    }      
  }
	
  jetiEx.SetJetiboxText( JetiExProtocol::LINE1, cLine1 );
  jetiEx.SetJetiboxText( JetiExProtocol::LINE2, cLine2 );
}

